@@ -53,11 +53,11 @@ import tempfile
 import smtplib
 import datetime
 import re
-from email.MIMEMultipart import MIMEMultipart
-from email.MIMEBase import MIMEBase
-from email.MIMEText import MIMEText
-from email.Utils import formatdate
-from email import Encoders
+import email.mime.multipart
+import email.mime.base
+import email.mime.text
+import email.utils
+import email.encoders
 import gzip
 import base64
 try:
@@ -73,10 +73,7 @@ except ImportError:
     import md5
     _hashfn = md5.md5
 
-try:
-    import simplejson as json
-except:
-    import json
+from boto.compat import json
 
 # List of Query String Arguments of Interest
 qsa_of_interest = ['acl', 'cors', 'defaultObjectAcl', 'location', 'logging',
@@ -86,7 +83,14 @@ qsa_of_interest = ['acl', 'cors', 'defaultObjectAcl', 'location', 'logging',
                    'response-content-language', 'response-expires',
                    'response-cache-control', 'response-content-disposition',
                    'response-content-encoding', 'delete', 'lifecycle',
-                   'tagging']
+                   'tagging', 'restore',
+                   # storageClass is a QSA for buckets in Google Cloud Storage.
+                   # (StorageClass is associated to individual keys in S3, but
+                   # having it listed here should cause no problems because
+                   # GET bucket?storageClass is not part of the S3 API.)
+                   'storageClass',
+                   # websiteConfig is a QSA for buckets in Google Cloud Storage.
+                   'websiteConfig']
 
 
 _first_cap_regex = re.compile('(.)([A-Z][a-z]+)')
@@ -100,9 +104,12 @@ def unquote_v(nv):
     else:
         return (nv[0], urllib.unquote(nv[1]))
 
-# generates the aws canonical string for the given parameters
+
 def canonical_string(method, path, headers, expires=None,
                      provider=None):
+    """
+    Generates the aws canonical string for the given parameters
+    """
     if not provider:
         provider = boto.provider.get_default()
     interesting_headers = {}
@@ -110,7 +117,7 @@ def canonical_string(method, path, headers, expires=None,
         lk = key.lower()
         if headers[key] != None and (lk in ['content-md5', 'content-type', 'date'] or
                                      lk.startswith(provider.header_prefix)):
-            interesting_headers[lk] = headers[key].strip()
+            interesting_headers[lk] = str(headers[key]).strip()
 
     # these keys get empty strings if they don't exist
     if 'content-type' not in interesting_headers:
@@ -154,6 +161,7 @@ def canonical_string(method, path, headers, expires=None,
 
     return buf
 
+
 def merge_meta(headers, metadata, provider=None):
     if not provider:
         provider = boto.provider.get_default()
@@ -169,6 +177,7 @@ def merge_meta(headers, metadata, provider=None):
 
     return final_headers
 
+
 def get_aws_metadata(headers, provider=None):
     if not provider:
         provider = boto.provider.get_default()
@@ -184,10 +193,21 @@ def get_aws_metadata(headers, provider=None):
             del headers[hkey]
     return metadata
 
+
 def retry_url(url, retry_on_404=True, num_retries=10):
+    """
+    Retry a url.  This is specifically used for accessing the metadata
+    service on an instance.  Since this address should never be proxied
+    (for security reasons), we create a ProxyHandler with a NULL
+    dictionary to override any proxy settings in the environment.
+    """
     for i in range(0, num_retries):
         try:
+            proxy_handler = urllib2.ProxyHandler({})
+            opener = urllib2.build_opener(proxy_handler)
             req = urllib2.Request(url)
+            r = opener.open(req)
+            result = r.read()
             resp = urllib2.urlopen(req)
             return resp.read()
         except urllib2.HTTPError, e:
@@ -203,13 +223,15 @@ def retry_url(url, retry_on_404=True, num_retries=10):
         except Exception, e:
             pass
         boto.log.exception('Caught exception reading instance data')
-        time.sleep(2**i)
+        time.sleep(2 ** i)
     boto.log.error('Unable to read instance data, giving up')
     return ''
 
+
 def _get_instance_metadata(url, num_retries):
     return LazyLoadMetadata(url, num_retries)
 
+
 class LazyLoadMetadata(dict):
     def __init__(self, url, num_retries):
         self._url = url
@@ -287,6 +309,7 @@ class LazyLoadMetadata(dict):
         self._materialize()
         return super(LazyLoadMetadata, self).__repr__()
 
+
 def get_instance_metadata(version='latest', url='http://169.254.169.254',
                           timeout=None, num_retries=5):
     """
@@ -312,6 +335,7 @@ def get_instance_metadata(version='latest', url='http://169.254.169.254',
         if timeout is not None:
             socket.setdefaulttimeout(original)
 
+
 def get_instance_identity(version='latest', url='http://169.254.169.254',
                           timeout=None, num_retries=5):
     """
@@ -338,6 +362,7 @@ def get_instance_identity(version='latest', url='http://169.254.169.254',
         if timeout is not None:
             socket.setdefaulttimeout(original)
 
+
 def get_instance_userdata(version='latest', sep=None,
                           url='http://169.254.169.254'):
     ud_url = '%s/%s/user-data' % (url, version)
@@ -354,11 +379,13 @@ def get_instance_userdata(version='latest', sep=None,
 ISO8601 = '%Y-%m-%dT%H:%M:%SZ'
 ISO8601_MS = '%Y-%m-%dT%H:%M:%S.%fZ'
 
+
 def get_ts(ts=None):
     if not ts:
         ts = time.gmtime()
     return time.strftime(ISO8601, ts)
 
+
 def parse_ts(ts):
     ts = ts.strip()
     try:
@@ -368,6 +395,7 @@ def parse_ts(ts):
         dt = datetime.datetime.strptime(ts, ISO8601_MS)
         return dt
 
+
 def find_class(module_name, class_name=None):
     if class_name:
         module_name = "%s.%s" % (module_name, class_name)
@@ -384,6 +412,7 @@ def find_class(module_name, class_name=None):
     except:
         return None
 
+
 def update_dme(username, password, dme_id, ip_address):
     """
     Update your Dynamic DNS record with DNSMadeEasy.com
@@ -393,6 +422,7 @@ def update_dme(username, password, dme_id, ip_address):
     s = urllib2.urlopen(dme_url % (username, password, dme_id, ip_address))
     return s.read()
 
+
 def fetch_file(uri, file=None, username=None, password=None):
     """
     Fetch a file based on the URI provided. If you do not pass in a file pointer
@@ -406,7 +436,8 @@ def fetch_file(uri, file=None, username=None, password=None):
     try:
         if uri.startswith('s3://'):
             bucket_name, key_name = uri[len('s3://'):].split('/', 1)
-            c = boto.connect_s3(aws_access_key_id=username, aws_secret_access_key=password)
+            c = boto.connect_s3(aws_access_key_id=username,
+                                aws_secret_access_key=password)
             bucket = c.get_bucket(bucket_name)
             key = bucket.get_key(key_name)
             key.get_contents_to_file(file)
@@ -426,20 +457,23 @@ def fetch_file(uri, file=None, username=None, password=None):
         file = None
     return file
 
+
 class ShellCommand(object):
 
-    def __init__(self, command, wait=True, fail_fast=False, cwd = None):
+    def __init__(self, command, wait=True, fail_fast=False, cwd=None):
         self.exit_code = 0
         self.command = command
         self.log_fp = StringIO.StringIO()
         self.wait = wait
         self.fail_fast = fail_fast
-        self.run(cwd = cwd)
+        self.run(cwd=cwd)
 
     def run(self, cwd=None):
         boto.log.info('running:%s' % self.command)
-        self.process = subprocess.Popen(self.command, shell=True, stdin=subprocess.PIPE,
-                                        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+        self.process = subprocess.Popen(self.command, shell=True,
+                                        stdin=subprocess.PIPE,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE,
                                         cwd=cwd)
         if(self.wait):
             while self.process.poll() == None:
@@ -468,6 +502,7 @@ class ShellCommand(object):
 
     output = property(getOutput, setReadOnly, None, 'The STDIN and STDERR output of the command')
 
+
 class AuthSMTPHandler(logging.handlers.SMTPHandler):
     """
     This class extends the SMTPHandler in the standard Python logging module
@@ -482,14 +517,16 @@ class AuthSMTPHandler(logging.handlers.SMTPHandler):
     args=('localhost', 'username', 'password', 'from@abc', ['user1@abc', 'user2@xyz'], 'Logger Subject')
     """
 
-    def __init__(self, mailhost, username, password, fromaddr, toaddrs, subject):
+    def __init__(self, mailhost, username, password,
+                 fromaddr, toaddrs, subject):
         """
         Initialize the handler.
 
         We have extended the constructor to accept a username/password
         for SMTP authentication.
         """
-        logging.handlers.SMTPHandler.__init__(self, mailhost, fromaddr, toaddrs, subject)
+        logging.handlers.SMTPHandler.__init__(self, mailhost, fromaddr,
+                                              toaddrs, subject)
         self.username = username
         self.password = password
 
@@ -512,7 +549,7 @@ class AuthSMTPHandler(logging.handlers.SMTPHandler):
                             self.fromaddr,
                             ','.join(self.toaddrs),
                             self.getSubject(record),
-                            formatdate(), msg)
+                            email.utils.formatdate(), msg)
             smtp.sendmail(self.fromaddr, self.toaddrs, msg)
             smtp.quit()
         except (KeyboardInterrupt, SystemExit):
@@ -520,6 +557,7 @@ class AuthSMTPHandler(logging.handlers.SMTPHandler):
         except:
             self.handleError(record)
 
+
 class LRUCache(dict):
     """A dictionary-like object that stores only a certain number of items, and
     discards its least recently used item when full.
@@ -553,11 +591,9 @@ class LRUCache(dict):
     C
 
     This code is based on the LRUCache class from Genshi which is based on
-    Mighty's LRUCache from ``myghtyutils.util``, written
-    by Mike Bayer and released under the MIT license (Genshi uses the
-    BSD License). See:
-
-      http://svn.myghty.org/myghtyutils/trunk/lib/myghtyutils/util.py
+    `Myghty <http://www.myghty.org>`_'s LRUCache from ``myghtyutils.util``,
+    written by Mike Bayer and released under the MIT license (Genshi uses the
+    BSD License).
     """
 
     class _Item(object):
@@ -565,6 +601,7 @@ class LRUCache(dict):
             self.previous = self.next = None
             self.key = key
             self.value = value
+
         def __repr__(self):
             return repr(self.value)
 
@@ -639,15 +676,18 @@ class LRUCache(dict):
         item.next = self.head
         self.head.previous = self.head = item
 
+
 class Password(object):
     """
     Password object that stores itself as hashed.
     Hash defaults to SHA512 if available, MD5 otherwise.
     """
-    hashfunc=_hashfn
+    hashfunc = _hashfn
+
     def __init__(self, str=None, hashfunc=None):
         """
-        Load the string from an initial value, this should be the raw hashed password.
+        Load the string from an initial value, this should be the
+        raw hashed password.
         """
         self.str = str
         if hashfunc:
@@ -670,7 +710,9 @@ class Password(object):
         else:
             return 0
 
-def notify(subject, body=None, html_body=None, to_string=None, attachments=None, append_instance_id=True):
+
+def notify(subject, body=None, html_body=None, to_string=None,
+           attachments=None, append_instance_id=True):
     attachments = attachments or []
     if append_instance_id:
         subject = "[%s] %s" % (boto.config.get_value("Instance", "instance-id"), subject)
@@ -679,20 +721,20 @@ def notify(subject, body=None, html_body=None, to_string=None, attachments=None,
     if to_string:
         try:
             from_string = boto.config.get_value('Notification', 'smtp_from', 'boto')
-            msg = MIMEMultipart()
+            msg = email.mime.multipart.MIMEMultipart()
             msg['From'] = from_string
             msg['Reply-To'] = from_string
             msg['To'] = to_string
-            msg['Date'] = formatdate(localtime=True)
+            msg['Date'] = email.utils.formatdate(localtime=True)
             msg['Subject'] = subject
 
             if body:
-                msg.attach(MIMEText(body))
+                msg.attach(email.mime.text.MIMEText(body))
 
             if html_body:
-                part = MIMEBase('text', 'html')
+                part = email.mime.base.MIMEBase('text', 'html')
                 part.set_payload(html_body)
-                Encoders.encode_base64(part)
+                email.encoders.encode_base64(part)
                 msg.attach(part)
 
             for part in attachments:
@@ -720,6 +762,7 @@ def notify(subject, body=None, html_body=None, to_string=None, attachments=None,
         except:
             boto.log.exception('notify failed')
 
+
 def get_utf8_value(value):
     if not isinstance(value, str) and not isinstance(value, unicode):
         value = str(value)
@@ -728,6 +771,7 @@ def get_utf8_value(value):
     else:
         return value
 
+
 def mklist(value):
     if not isinstance(value, list):
         if isinstance(value, tuple):
@@ -736,6 +780,7 @@ def mklist(value):
             value = [value]
     return value
 
+
 def pythonize_name(name):
     """Convert camel case to a "pythonic" name.
 
@@ -772,17 +817,17 @@ def write_mime_multipart(content, compress=False, deftype='text/plain', delimite
     :return: Final mime multipart
     :rtype: str:
     """
-    wrapper = MIMEMultipart()
+    wrapper = email.mime.multipart.MIMEMultipart()
     for name, con in content:
         definite_type = guess_mime_type(con, deftype)
         maintype, subtype = definite_type.split('/', 1)
         if maintype == 'text':
-            mime_con = MIMEText(con, _subtype=subtype)
+            mime_con = email.mime.text.MIMEText(con, _subtype=subtype)
         else:
-            mime_con = MIMEBase(maintype, subtype)
+            mime_con = email.mime.base.MIMEBase(maintype, subtype)
             mime_con.set_payload(con)
             # Encode the payload using Base64
-            Encoders.encode_base64(mime_con)
+            email.encoders.encode_base64(mime_con)
         mime_con.add_header('Content-Disposition', 'attachment', filename=name)
         wrapper.attach(mime_con)
     rcontent = wrapper.as_string()
@@ -798,6 +843,7 @@ def write_mime_multipart(content, compress=False, deftype='text/plain', delimite
 
     return rcontent
 
+
 def guess_mime_type(content, deftype):
     """Description: Guess the mime type of a block of text
     :param content: content we're finding the type of
@@ -810,13 +856,13 @@ def guess_mime_type(content, deftype):
     :return: <description>
     """
     #Mappings recognized by cloudinit
-    starts_with_mappings={
-        '#include' : 'text/x-include-url',
-        '#!' : 'text/x-shellscript',
-        '#cloud-config' : 'text/cloud-config',
-        '#upstart-job'  : 'text/upstart-job',
-        '#part-handler' : 'text/part-handler',
-        '#cloud-boothook' : 'text/cloud-boothook'
+    starts_with_mappings = {
+        '#include': 'text/x-include-url',
+        '#!': 'text/x-shellscript',
+        '#cloud-config': 'text/cloud-config',
+        '#upstart-job': 'text/upstart-job',
+        '#part-handler': 'text/part-handler',
+        '#cloud-boothook': 'text/cloud-boothook'
     }
     rtype = deftype
     for possible_type, mimetype in starts_with_mappings.items():
@@ -825,6 +871,7 @@ def guess_mime_type(content, deftype):
             break
     return(rtype)
 
+
 def compute_md5(fp, buf_size=8192, size=None):
     """
     Compute MD5 hash on passed file and return results in a tuple of values.
